#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function

import os
import json
import socket
import SocketServer
import argparse

from cts_auth.systemd_shim import handle_systemd, check_svc_shim
from cts_auth.pam_shim import auth, check_auth_shim

__DEFAULT_SOCK = "/var/run/ctsauth/pam.sock"


def process_cmd(cmd):
    try:
        cmd = json.loads(cmd)

        if cmd.get('systemd'):
            return json.dumps(handle_systemd(cmd))

        return json.dumps(auth(cmd))

    except Exception as ex:
        print(ex)


def test_socket(msg):
    if os.path.exists(__DEFAULT_SOCK):
        client = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        client.connect(__DEFAULT_SOCK)
        client.send(msg)
        resp = client.recv(1024)
        client.close()
        return resp
    else:
        print("Couldn't Connect!")


class ThreadedRequestHandler(SocketServer.BaseRequestHandler):
    def handle(self):
        ret = process_cmd(self.request.recv(1024))
        if ret:
            self.request.sendall(ret)


class ThreadedServer(SocketServer.ThreadingMixIn, SocketServer.UnixStreamServer):
    SYSTEMD_FIRST_SOCKET_FD = 3

    def __init__(self, server_address, handler_cls):
        SocketServer.UnixStreamServer.__init__(self, server_address, handler_cls, bind_and_activate=False)
        self.socket = socket.fromfd(self.SYSTEMD_FIRST_SOCKET_FD, self.address_family, self.socket_type)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--username', type=str, dest='username', help='username')
    parser.add_argument('--password', type=str, dest='password', help='password')
    parser.add_argument('--service', type=str, dest='service', help='service')
    parser.add_argument('--action', type=str, dest='action', help='action')
    args = parser.parse_args()

    if args.username and args.password:
        check_auth_shim(test_socket, username=args.username, password=args.password)
    elif args.service and args.action:
        check_svc_shim(test_socket, action=args.action, service=args.service)
    else:
        server = ThreadedServer(None, ThreadedRequestHandler)
        server.serve_forever()


if __name__ == "__main__":
    main()
