import os
import json
from subprocess import check_output, CalledProcessError


def safe_run_checkoutput(cmd):
    with open(os.devnull, 'w') as fnull:
        try:
            return 0, check_output(cmd, stderr=fnull)
        except CalledProcessError as cpe:
            return cpe.returncode, cpe.output
        except Exception as ex:
            return -1, ex.message


def systemd_service(cmd, service):
    '''Common systemctl commands function for daemons'''
    legit_cmds = ['start', 'stop', 'status', 'restart', 'reload', 'disable', 'enable', 'is-active']
    systemctl = '/bin/systemctl'

    if cmd not in legit_cmds:
        raise NotImplemented('systemd command %s not recognized.' % cmd)

    if cmd == 'status':
        return safe_run_checkoutput([systemctl, cmd, service])
    else:
        # ctsauth has been restricted to manipulating only tips-appliance as sudo
        return safe_run_checkoutput(['sudo',systemctl, cmd, service])


def handle_systemd(cmd):
    try:
        service = cmd['service']
        action = cmd['action']
        status, resp = systemd_service(action, service)
        return {
            'cmd': cmd,
            'service': service,
            'status': status,
            'response': resp
        }
    except Exception as ex:
        print(ex)


def check_svc_shim(sink, action, service):
    src = json.dumps({'systemd':True,'action': action, 'service': service}).encode('utf-8')
    try:
        resp = sink(src)
        print(resp)
    except Exception as ex:
        print(ex)
